/* UTCSelectForm.cls */

USING Infragistics.Win.Misc.* FROM ASSEMBLY.
USING System.Windows.Forms.* FROM ASSEMBLY.
USING Progress.Util.* FROM ASSEMBLY.

CLASS UTCSelectForm INHERITS Progress.Windows.Form: 

  /* Variables for controls on form */
  DEFINE PRIVATE VARIABLE rTimeButton AS CLASS UltraButton NO-UNDO.
  DEFINE PRIVATE VARIABLE rTZButton AS CLASS UltraButton NO-UNDO.
  DEFINE PRIVATE VARIABLE rDateField AS CLASS UltraLabel NO-UNDO.
  DEFINE PRIVATE VARIABLE lUTCSelected AS LOGICAL INITIAL FALSE NO-UNDO.
  
  /* Public properties and methods */
  METHOD PUBLIC VOID DoWait( ):
    WAIT-FOR Application:Run( THIS-OBJECT ).
  END METHOD.

  /* Constructor and destructor */
  CONSTRUCTOR PUBLIC UTCSelectForm( ):    
    InitializeComponent( ).
  END CONSTRUCTOR.
  
  /* Private form initialization method */
  METHOD PRIVATE VOID InitializeComponent( ):
  
    /* Creates base form class and all components */
    rTimeButton = NEW UltraButton( ).
    rTZButton = NEW UltraButton( ).
    rDateField = NEW UltraLabel( ).
    
    THIS-OBJECT:SuspendLayout( ).
    
    /* Initialize current date/time field */
    rDateField:Text = GetTime( ).
    rDateField:Size = TextRenderer:MeasureText( rDateField:Text, 
                                                rDateField:Font ).
    rDateField:Top = 20.
    
    /* Initialize the Time button */
    rTimeButton:Text = "Time".
    rTimeButton:Size = TextRenderer:MeasureText( rTimeButton:Text, 
                                                 rTimeButton:Font ).
    rTimeButton:Height = rTimeButton:Height * 2.
    rTimeButton:Width = rTimeButton:Width * 1.5.
    rTimeButton:Top = rDateField:Top + rDateField:Height + 10.
    rTimeButton:Click:Subscribe( TimeButton_Click ).
    
    /* Initialize the Close button */
    rTZButton:Text = "Select Zone".
    rTZButton:Size = TextRenderer:MeasureText( rTZButton:Text, 
                                               rTZButton:Font ).
    rTZButton:Height = rTimeButton:Height.
    rTZButton:Width = rTZButton:Width * 1.5.
    rTZButton:Top = rTimeButton:Top.
    rTZButton:Click:Subscribe( TZButton_Click ).
    
    /* Initialize the form with its field and buttons */
    THIS-OBJECT:Text = "Current Date and Time".
    THIS-OBJECT:FormBorderStyle = FormBorderStyle:FixedSingle.
    THIS-OBJECT:MaximizeBox = FALSE.
    THIS-OBJECT:Controls:Add( rDateField ).
    THIS-OBJECT:Controls:Add( rTimeButton ).
    THIS-OBJECT:Controls:Add( rTZButton ).
    THIS-OBJECT:AcceptButton = rTimeButton.
    
    /* Adjust form size and the location for field and button */
    THIS-OBJECT:Width = rDateField:Width * 2.
    THIS-OBJECT:Height = rTZButton:Top + rTZButton:Height 
                         + rDateField:Height + 40.
    rDateField:Left = ( THIS-OBJECT:Width - rDateField:Width ) / 2.
    rTimeButton:Left = ( THIS-OBJECT:Width - ( rTimeButton:Width + 10 
                                           + rTZButton:Width ) ) / 2.
    rTZButton:Left = rTimeButton:Left + rTimeButton:Width + 10.

    THIS-OBJECT:ResumeLayout( FALSE ).
  
  END METHOD.
  
  /* Private form event handlers */
  METHOD PRIVATE VOID TimeButton_Click( sender AS System.Object, 
                                        e AS System.EventArgs ):
    rDateField:Text = GetTime( ).
    rDateField:Size = TextRenderer:MeasureText( rDateField:Text, 
                                                rDateField:Font ).
  END METHOD.

  METHOD PRIVATE VOID TZButton_Click( sender AS System.Object, 
                                      e AS System.EventArgs ):
    DEFINE VARIABLE rDialog AS CLASS UTCDialog NO-UNDO.
    DEFINE VARIABLE enResult AS CLASS DialogResult NO-UNDO.
    rDialog = NEW UTCDialog( lUTCSelected ).
    rDialog:WaitForDialog( INPUT THIS-OBJECT, OUTPUT enResult ).
    
    rDialog:Dispose( ).

    IF EnumHelper:AreEqual( enResult, DialogResult:Yes )
    THEN
      lUTCSelected = TRUE.
    ELSE
      lUTCSelected = FALSE.
    
    rDateField:Text = GetTime( ).
    rDateField:Size = TextRenderer:MeasureText( rDateField:Text, 
                                                rDateField:Font ).
  END METHOD.
  
  METHOD PRIVATE CHARACTER GetTime( ):
    DEFINE VARIABLE dTime AS DATETIME NO-UNDO.
    DEFINE VARIABLE iHour AS INTEGER INITIAL 3600000 NO-UNDO.
    
    dTime = NOW.
    IF lUTCSelected
    THEN DO:
      dTime = dTime + (5 * iHour).
      RETURN STRING(dTime) + " UTC".
    END.
    ELSE DO:
      RETURN STRING(dTime) + " EST".
    END.
  END METHOD.
    
END CLASS.
