/* MultipleBindings.p 
   Bind two BindingSource objects to the same ProDataSet then display the parent/child records in separate grids. */

/* USING statements must be the first in the procedure */
USING System.Windows.Forms.*.
USING Infragistics.Win.UltraWinGrid.*.

DEFINE VARIABLE rMainForm  AS Progress.Windows.Form NO-UNDO.
DEFINE VARIABLE rCustGrid  AS UltraGrid NO-UNDO.
DEFINE VARIABLE rInvGrid   AS UltraGrid NO-UNDO.
DEFINE VARIABLE rCustBindS AS Progress.Data.BindingSource NO-UNDO.
DEFINE VARIABLE rInvBindS  AS Progress.Data.BindingSource NO-UNDO.
DEFINE VARIABLE rControls  AS Control+ControlCollection NO-UNDO.

DEFINE VARIABLE ttCustomerHdl AS HANDLE NO-UNDO.
DEFINE VARIABLE ttInvoiceHdl  AS HANDLE NO-UNDO.

DEFINE VARIABLE hDataSet   AS HANDLE NO-UNDO.
DEFINE VARIABLE hBufTTCust AS HANDLE NO-UNDO.
DEFINE VARIABLE hBufTTInv  AS HANDLE NO-UNDO.
DEFINE VARIABLE hCustQry   AS HANDLE NO-UNDO.
DEFINE VARIABLE hInvQry    AS HANDLE NO-UNDO.

DEFINE TEMP-TABLE ttCustomer NO-UNDO LIKE Customer.
DEFINE TEMP-TABLE ttInvoice NO-UNDO  LIKE Invoice.

DEFINE DATASET dsCustInv FOR ttCustomer, ttInvoice
  DATA-RELATION FOR ttCustomer, ttInvoice RELATION-FIELDS(CustNum, CustNum).
	   
hDataSet = DATASET dsCustInv:HANDLE.

ttCustomerHdl = TEMP-TABLE ttCustomer:HANDLE.
ttInvoiceHdl = TEMP-TABLE ttInvoice:HANDLE.

hBufTTCust = ttCustomerHdl:DEFAULT-BUFFER-HANDLE.
hBufTTInv = ttInvoiceHdl:DEFAULT-BUFFER-HANDLE.

/* Define the data-sources */
DEFINE DATA-SOURCE dCust FOR Customer.
DEFINE DATA-SOURCE dInv FOR Invoice.

/* Attach the data-sources to the dataset buffers */
hBufTTCust:ATTACH-DATA-SOURCE(DATA-SOURCE dCust:HANDLE,?,?,?).
hBufTTInv:ATTACH-DATA-SOURCE(DATA-SOURCE dInv:HANDLE,?,?,?).

/* Fill the dataset using the data-sources */
hDataSet:FILL().

/* Navigation query for Customer */
hCustQry = hDataSet:TOP-NAV-QUERY().
hCustQry:QUERY-PREPARE("PRESELECT EACH ttCustomer").
hCustQry:QUERY-OPEN().  
hCustQry:GET-NEXT(). 

/* Navigation query for Invoice */
hInvQry = hDataSet:GET-TOP-BUFFER():GET-CHILD-RELATION():QUERY.
hInvQry:QUERY-OPEN().

/* Display all Customer fields in the grid. */
rCustBindS = NEW Progress.Data.BindingSource(hCustQry, "*", "").
/* Display all Invoice fields in the grid. */
rInvBindS = NEW Progress.Data.BindingSource(hInvQry, "*", "").

/* Disable editing because procedure does not include event logic to handle changes. */
rCustBindS:AllowEdit = FALSE.
rCustBindS:AllowRemove = FALSE.
rInvBindS:AllowEdit = FALSE.
rInvBindS:AllowRemove = FALSE.

/* Main block */
IF VALID-OBJECT(rCustBindS) AND VALID-OBJECT(rInvBindS) THEN
  DO ON ERROR UNDO, LEAVE:

    rMainForm = NEW Progress.Windows.Form().
    rMainForm:Width = 900.
    rMainForm:Height = 520.
    rMainForm:Text = "Customer & Invoice Form".

    rCustGrid = NEW UltraGrid().
    rCustGrid:Left = 10.
    rCustGrid:Top = 10.
    rCustGrid:Width = 860.
    rCustGrid:Height = 250.
    rCustGrid:Name = "CustGrid".
    rCustGrid:Text = "Customer Grid".
    rCustGrid:DataSource = rCustBindS.
    rCustGrid:TabIndex = 1.

    rInvGrid = NEW UltraGrid().
    rInvGrid:Left = 10.
    rInvGrid:Top = 270.
    rInvGrid:Width = 860.
    rInvGrid:Height = 200.
    rInvGrid:Name = "InvGrid".
    rInvGrid:Text = "Invoice Grid".
    rInvGrid:DataSource = rInvBindS.
    rInvGrid:TabIndex = 2.

    rControls = rMainForm:Controls.
    rControls:Add(rCustGrid).
    rControls:Add(rInvGrid).

    rCustBindS:PositionChanged:SUBSCRIBE("CustPositionChanged").

    WAIT-FOR Application:RUN(rMainForm).
    RUN cleanup.

  END. /* Main block */

PROCEDURE CustPositionChanged:
/* When Position changes in CustGrid, synchronize and refresh InvGrid. 
   Alternately, you could set the buffer's Auto-Synchronize attribute to TRUE.*/  
  DEFINE INPUT PARAMETER rSender AS Progress.Data.BindingSource NO-UNDO.
  DEFINE INPUT PARAMETER rArgs AS System.EventArgs NO-UNDO.

  hBufTTCust:SYNCHRONIZE().
  rInvBindS:RefreshAll().

END PROCEDURE.

PROCEDURE cleanup:
/* Delete all objects to avoid memory leaks */

  rInvBindS:Dispose( ).
  rCustBindS:Dispose( ).

END PROCEDURE.


