/* h-cleanup.p -- shows how to delete a dynamic query and its dynamic buffers. */

DEFINE VARIABLE cBufHandles AS CHARACTER  NO-UNDO.
/* A list of tables for the dynamic query comes in as a parameter or is
   determined at run time . This simulates the list of names. */
DEFINE VARIABLE cBufNames   AS CHARACTER  NO-UNDO
  INITIAL "Customer,Order,SalesRep".
DEFINE VARIABLE hBuffer     AS HANDLE     NO-UNDO.
DEFINE VARIABLE hQuery      AS HANDLE     NO-UNDO.
DEFINE VARIABLE iBufNum     AS INTEGER    NO-UNDO.

/* Create a dynamic query and dynamic buffers for each table. Add each buffer
   to the query. */
CREATE QUERY hQuery.
DO iBufNum = 1 TO NUM-ENTRIES(cBufNames):
  CREATE BUFFER hBuffer FOR TABLE ENTRY(iBufNum, cBufNames).
  hQuery:ADD-BUFFER(hBuffer).
END.

/* Verify that the query has the buffers we created. */
cBufNames = "". 
DO iBufNum = 1 TO hQuery:NUM-BUFFERS:
  cBufNames = cBufNames + hQuery:GET-BUFFER-HANDLE(iBufNum):NAME + " ".
END.
MESSAGE "This query uses buffers " cBufNames VIEW-AS ALERT-BOX.

/* Application code here uses the dynamic query and its buffers...  */
/* When you're done with the query, build up a list of its buffers. */
DO iBufNum = 1 TO hQuery:NUM-BUFFERS:
  cBufHandles = cBufHandles + (IF cBufHandles = "" THEN "" ELSE ",") +
    STRING(hQuery:GET-BUFFER-HANDLE(iBufNum)).
END.

MESSAGE "Buffer handles are: " cBufHandles VIEW-AS ALERT-BOX.

/* After building the list, delete the buffers and then the query. */
DO iBufNum = 1 TO hQuery:NUM-BUFFERS:
  DELETE OBJECT HANDLE(ENTRY(iBufNum, cBufHandles)).
END.

DELETE OBJECT hQuery.
