/* h-TestDynBrowse.p -- test dynamic temp-table methods. */

DEFINE VARIABLE hTT     AS HANDLE             NO-UNDO.
DEFINE VARIABLE hTTBuf  AS HANDLE             NO-UNDO.
DEFINE VARIABLE hQuery  AS HANDLE             NO-UNDO.
DEFINE VARIABLE iSeq    AS INTEGER            NO-UNDO INITIAL 1000.
DEFINE VARIABLE cName   LIKE Customer.NAME    NO-UNDO.
DEFINE VARIABLE iCNum   LIKE Customer.CustNum NO-UNDO.
DEFINE VARIABLE cRep    LIKE SalesRep.RepName NO-UNDO.
DEFINE VARIABLE hBrowse AS HANDLE             NO-UNDO.
DEFINE VARIABLE hColumn AS HANDLE             NO-UNDO.

DEFINE FRAME BrowseFrame WITH SIZE 80 BY 10.

CREATE TEMP-TABLE hTT.
hTT:CREATE-LIKE("Customer","Name").
hTT:ADD-FIELDS-FROM("SalesRep","MonthQuota").
hTT:ADD-LIKE-FIELD("Area", "SalesRep.Region").
hTT:ADD-NEW-FIELD("Sequence", "INTEGER",0,"9999",1000).
hTT:ADD-LIKE-INDEX("CustNum","CustNum","Customer").
hTT:ADD-NEW-INDEX("SeqIndex", TRUE).  /* Make it the primary index */
hTT:ADD-INDEX-FIELD("SeqIndex", "Sequence").
hTT:TEMP-TABLE-PREPARE("CustSequence").
hTTBuf = hTT:DEFAULT-BUFFER-HANDLE.

/* Populate the temp-table with values from the database. */
FOR EACH Customer WHERE Customer.State = "NH", SalesRep OF Customer:
  hTTBuf:BUFFER-CREATE().
  hTTBuf:BUFFER-COPY(BUFFER Customer:HANDLE).
  hTTBuf:BUFFER-COPY(BUFFER SalesRep:HANDLE).
  hTTBuf:BUFFER-FIELD("Sequence"):BUFFER-VALUE = iSeq.
  iSeq = iSeq + 1.
END.

/* Create a query for the temp-table buffer and display values. */
CREATE QUERY hQuery.
hQuery:SET-BUFFERS(hTTBuf).
hQuery:QUERY-PREPARE("FOR EACH CustSequence").

CREATE BROWSE hBrowse ASSIGN 
  ROW         = 1
  COL         = 1
  WIDTH       = 79
  DOWN        = 10
  FRAME       = FRAME BrowseFrame:HANDLE
  QUERY       = hQuery
  SENSITIVE   = TRUE
  SEPARATORS  = TRUE
  ROW-MARKERS = FALSE
  VISIBLE     = TRUE
  TRIGGERS:
    ON ROW-DISPLAY
      hColumn:SCREEN-VALUE =
        STRING(hTTBuf:BUFFER-FIELD("CreditLimit"):BUFFER-VALUE -
        hTTBuf:BUFFER-FIELD("Balance"):BUFFER-VALUE).
  END.

/* hBrowse:ADD-COLUMNS-FROM(hTTBuf, "SalesRep,Country,address,Address2,State").  */
hBrowse:ADD-LIKE-COLUMN(hTTBuf:BUFFER-FIELD("Sequence")).
hBrowse:ADD-LIKE-COLUMN(hTTBuf:BUFFER-FIELD("CustNum")).
hBrowse:ADD-LIKE-COLUMN(hTTBuf:BUFFER-FIELD("Name")).
hBrowse:ADD-LIKE-COLUMN(hTTBuf:BUFFER-FIELD("RepName")).
/* hTTBuf:BUFFER-FIELD("SalesRep"):VALIDATE-EXPRESSION = "".           */
/* hColumn = hBrowse:ADD-LIKE-COLUMN(hTTBuf:BUFFER-FIELD("SalesRep")). */
hColumn = hBrowse:ADD-CALC-COLUMN("DECIMAL","ZZ,ZZZ,ZZ9.99",0,"Available",4).

hQuery:QUERY-OPEN().
ENABLE ALL WITH FRAME BrowseFrame.
WAIT-FOR CLOSE OF CURRENT-WINDOW.

