/* r-WaitForms.cls */

USING System.Windows.Forms.* FROM ASSEMBLY.
USING Progress.Util.* FROM ASSEMBLY.

CLASS r-WaitForms INHERITS Progress.Windows.Form:
  DEFINE PRIVATE VARIABLE rFormDescr AS CLASS Label NO-UNDO.

  METHOD PUBLIC VOID DoWait( ).
    /* Display and wait for the non-modal form to close */
    WAIT-FOR Application:Run( INPUT THIS-OBJECT ).
  END METHOD.

  CONSTRUCTOR PUBLIC r-WaitForms( ):
    /* Initialize and subscribe to events */
    InitializeComponent( ).
    THIS-OBJECT:FormClosing:Subscribe(Form_Closing).
  END CONSTRUCTOR.

  METHOD PRIVATE VOID InitializeComponent( ):
    /* Initialize the non-modal form class and components */
    rFormDescr = NEW Label( ).

    /* Initialize the form description label */
    rFormDescr:Text     = "Click the Close (X) button of this form to pop-up a dialog box ...".
    rFormDescr:Size     = NEW System.Drawing.Size( INPUT 330, INPUT 13 ).
    rFormDescr:Location = NEW System.Drawing.Point( INPUT 4, INPUT 6 ).

    /* Initialize the non-modal form */
    THIS-OBJECT:FormBorderStyle = FormBorderStyle:FixedSingle.
    THIS-OBJECT:Text = "This is my form.".
    THIS-OBJECT:Controls:Add( INPUT rFormDescr ).
    THIS-OBJECT:Size = NEW System.Drawing.Size( INPUT rFormDescr:Width, INPUT 60 ).
  END METHOD.    

  METHOD PRIVATE VOID Form_Closing
    ( INPUT sender AS System.Object, INPUT e AS FormClosingEventArgs ):
    DEFINE VARIABLE rDialog        AS CLASS Progress.Windows.Form NO-UNDO.
    DEFINE VARIABLE rDialogDescr   AS CLASS Label  NO-UNDO.
    DEFINE VARIABLE rOKButton      AS CLASS Button NO-UNDO.
    DEFINE VARIABLE rCancelButton  AS CLASS Button NO-UNDO.
    DEFINE VARIABLE enDialogResult AS CLASS DialogResult NO-UNDO.

    /* Create dialog box components */
    ASSIGN
      rDialog       = NEW Progress.Windows.Form( )
      rDialogDescr  = NEW Label( )
      rOKButton     = NEW Button( )
      rCancelButton = NEW Button( ).

    /* Initialize the dialog description label */
    rDialogDescr:Text     = "Click OK to close form or click Cancel to leave form open.".
    rDialogDescr:Size     = NEW System.Drawing.Size( INPUT 306, INPUT 13).
    rDialogDescr:Location = NEW System.Drawing.Point( INPUT 4, INPUT 6 ).

    /* Initialize the buttons */
    rOKButton:Text = "OK".
    rOKButton:Size = NEW System.Drawing.Size( INPUT 60, INPUT 20).
    rOKButton:Location = NEW System.Drawing.Point
      ( INPUT INTEGER( ( rDialogDescr:Width - 124 ) / 2 ), 
        INPUT rDialogDescr:Top + rDialogDescr:Height + 8 ).
    rOKButton:DialogResult = DialogResult:OK.
    rCancelButton:Text = "Cancel".
    rCancelButton:Size = NEW System.Drawing.Size( INPUT 60, INPUT 20 ). 
    rCancelButton:Location = NEW System.Drawing.Point
      ( INPUT rOKButton:Left + rOKButton:Width + 4, 
        INPUT rDialogDescr:Top + rDialogDescr:Height + 8 ).
    rCancelButton:DialogResult = DialogResult:Cancel.

    /* Initialize the modal dialog box with label and buttons */
    rDialog:FormBorderStyle = FormBorderStyle:FixedDialog.
    rDialog:Controls:Add( INPUT rDialogDescr ).
    rDialog:Controls:Add( INPUT rOKButton ).
    rDialog:Controls:Add( INPUT rCancelButton ).
    rDialog:Text = "My form is closing ...".
    rDialog:Size = NEW System.Drawing.Size( INPUT 306, INPUT 106 ).

    /* Display dialog box to handle FormClosing event and the results */
    WAIT-FOR rDialog:ShowDialog( ) SET enDialogResult.

    IF EnumHelper:AreEqual
      ( INPUT enDialogResult, INPUT DialogResult:Cancel ) THEN DO:
      MessageBox:Show( INPUT "My form closing was canceled." ).
      e:Cancel = TRUE. /* Cancel FormClosing; leave main form open */
    END.
    ELSE DO:
      MessageBox:Show( INPUT "My form is closing OK." ).
      e:Cancel = FALSE. /* Continue FormClosing; close main form */
    END.

    rDialog:Dispose( ). /* Dispose modal form object */  
  END METHOD. /* Form_Closing */
END CLASS.
