/* i-sax1hn.p - Name attribute for the current entry. App gets it during the 
   StartElement callback */
DEFINE VARIABLE currentPerson AS CHARACTER NO-UNDO.
/* Phone number from the current entry. App gets it during the Characters
   callback because it is the character data for the element. */
DEFINE VARIABLE currentNum    AS CHARACTER NO-UNDO.

/* This procedure is called when the parser finds the start tag for an element.
  For this particular XML doc, the app simply looks for "Entry" elements and
  digs out the "ContactName" attribute during the StartElement call, saving it
  in currentPerson. 
  The code assumes that Namespace processing is enabled and checks
  to make sure that name parameters are part of the correct namespace 
  */
PROCEDURE StartElement:
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER localName    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER qName        AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER attributes   AS HANDLE    NO-UNDO.

  IF namespaceURI = "http://www.wmhwmh.biz/ns/phonelist" THEN DO:
    IF localName = "Entry" THEN
      currentPerson = attributes:GET-VALUE-BY-NAMESPACE-NAME
        ("http://www.wmhwmh.biz/ns/phonelist", "ContactName" ).
  END.
END.

/* This callback gets passed the character data for an element. SAX
   does not guarantee that all the characters for an element get passed in
   one call -- that's why the app has to maintain the currentNum global
   variable and append to it when handling Characters, and also why it has to
   wait for EndElement before displaying the message box. (Some apps may
   need to use a MEMPTR to accumulate the character data, which may
   exceed the 32K Progress CHARACTER variable limit) */
PROCEDURE Characters:
  DEFINE INPUT PARAMETER charData AS MEMPTR  NO-UNDO.
  DEFINE INPUT PARAMETER numChars AS INTEGER NO-UNDO.
  
  /* Can assume that any call to Characters is for an Entry's text value,
    because we know what the document looks like. If this weren't the case,
    we'd have to keep track of the localName passed to the most recent call to
    StartElement */
  currentNum = currentNum + GET-STRING(charData, 1, GET-SIZE(charData)).
END.

/* This callback is called when the parser finds the end tag for an Element.
   This app only cares about the end of an Entry element.*/
PROCEDURE EndElement:
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER localName    AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER qName        AS CHARACTER NO-UNDO.

  IF namespaceURI = "http://www.wmhwmh.biz/ns/phonelist" THEN DO:
    IF  localName = "Entry" THEN DO:
      MESSAGE "Name: " currentPerson SKIP 
        "Phone Number: " currentNum VIEW-AS ALERT-BOX.
      currentNum = "".
      currentPerson = "".
    END.
  END.
END.

/* Note that, knowing the structure of the XML doc, the app could have done
   this in the EndElement call for the Phonelist element and could then have
   omitted EndDocument altogether. */
PROCEDURE EndDocument:
  MESSAGE "All Done" VIEW-AS ALERT-BOX.
END.
