 /*------------------------------------------------------------------------
    File        : businessEntity
    Purpose     : 
    Syntax      : 
    Description : 
    Author(s)   : 
    Created     : Fri Jul 27 11:50:30 EDT 2007
    Notes       : 
  ----------------------------------------------------------------------*/
CLASS openedge.tutorial.services.BusinessEntity   : 
    
    def public property DatasetHandle as handle no-undo
        get.
        protected set.
        
    def public property DatasetName as character no-undo
        get.
        protected set.

    def private temp-table ttDataSource no-undo
        field EntityName as character
        field BufferHdl as handle
        field DataSource as handle. 
    
    method protected void AddTable (input phDataset as handle, 
                                    input phDataTable as handle,
                                    input phBuffer as handle, 
                                    input pcJoinFields as character,
                                    input phParentTable as handle):
                                      
        /* add buffer to PDS */
        phDataset:add-buffer(phDataTable:default-buffer-handle).
        
        AddDataSource(phDataset, phBuffer, phDataTable:default-buffer-handle).
        
        if pcJoinFields ne '' and pcJoinFields ne ? then
            phDataset:add-relation(phParentTable:default-buffer-handle,
                                   phDataTable:default-buffer-handle,
                                   pcJoinFields).
    end method.
    
    method protected void AddDataSource (input phDataset as handle,
                                         input phDbBuffer as handle,
                                         input phTTBuffer as handle   ):
                                                                                                                                 
        def var hDataSource as handle no-undo.
                                                   
        create data-source hDataSource.
        hDataSource:add-source-buffer(phDbBuffer, phDbBuffer:keys).
        
    	/* associate the dataset buffer handle with the datasource. */
        phTTBuffer:attach-data-source(hDataSource).
        
        create ttDataSource.
        ttDataSource.EntityName = phDataset:name.
        ttDataSource.BufferHdl = phTTBuffer.
        ttDataSource.DataSource = hDataSource.
    end method.        
    
    method public void saveData(input phDataSet as handle):        
        def var iBuf as int no-undo.
        def var iRow as int no-undo.
        def var hBuffer as handle no-undo.
        def var hBeforeBuffer as handle no-undo.
        def var hQry as handle no-undo.
        
        AttachDataSources(phDataset).
                
        create query hQry.
        do iBuf = 1 to phDataSet:num-buffers:
            hBuffer = phDataSet:get-buffer-handle(iBuf).
            hBeforeBuffer = hBuffer:before-buffer.
            
            hQry:set-buffers(hBeforeBuffer).
            hQry:query-prepare('for each ' + hBeforeBuffer:name).
            hQry:query-open().
            hQry:get-first().
            do transaction while not hQry:query-off-end:
                if hBeforeBuffer:row-state > 0 then
                    hBeforeBuffer:save-row-changes() no-error.
                hQry:get-next().
            end.
        end.    /* buffer loop */               
        
        delete object hQry no-error.
    end method.    /* savedata */    

    method public void AttachDataSources (input phDataset as handle):
        def buffer ttDataSource for ttDataSource.
        
        for each ttDataSource where ttDataSource.EntityName = phDataset:name:
            ttDataSource.BufferHdl:attach-data-source(ttDataSource.DataSource).
        end.
    end method.
    
    method public void DetachDataSources(input phDataset as handle):
        define variable hBuffer as handle    no-undo.
        define variable iBuffer as integer   no-undo.
        
        do iBuffer = 1 to phDataset:num-buffers:
            hBuffer = phDataset:get-buffer-handle(iBuffer).
            if valid-handle(hBuffer:data-source) then
                hBuffer:detach-data-source().
        end.
    end method. 
    
    method protected handle BuildBuffer(input pcTable as character):
    	/*------------------------------------------------------------------------------
		    Purpose: Builds a buffer
		    Notes:
		------------------------------------------------------------------------------*/
        define variable hBuffer as handle no-undo.
            
        create buffer hBuffer for table pcTable buffer-name pcTable no-error.
            
        return hBuffer.
    end method.    /* buildbuffer */
    
    method protected handle BuildDataTable ( input phBuffer as handle):
    	/*------------------------------------------------------------------------------
		    Purpose: Creates data table from buffer
		    Notes:
		------------------------------------------------------------------------------*/
        define variable hDataTable as handle no-undo.
        def var hBeforeTable as handle no-undo.
            
        if valid-handle(phBuffer) then
        do:
            create temp-table hDataTable.
            hDataTable:create-like(phBuffer).
            hDataTable:temp-table-prepare("e" + phBuffer:name).
        end.
                    
        return hDataTable.
    end method.    /* buildDataTable */            
    
    method public void FillEntity(plEnableTracking as logical):
        def var iTbl as integer no-undo.
        
        DatasetHandle:fill no-error.
            
        DetachDataSources(DatasetHandle).
        
        /* Turn on tracking */
        if plEnableTracking then
        do iTbl = 1 to DatasetHandle:num-buffers:
            DatasetHandle:get-buffer-handle(iTbl):table-handle:tracking-changes = yes.
        end.
    end method. /* fill */
    
    method public void RegisterCallbacks():
        /* abstract method */
    end method.
END CLASS.